module Hexena.Cmds.ClassifySVM (run) where

import Data.Binary.Get
import qualified Data.ByteString as B
import qualified Data.ByteString.Lazy as L
import Data.IFF
import qualified Data.IntMap as IM
import qualified Data.Map as M
import Data.Maybe (fromJust)
import System.IO (hPutStrLn, stderr)
import Text.Printf

--import Configuration -- Using Paths_hexena as generated by cabal
import Paths_hexena (getDataFileName)

import Hexena.Chunks
import Hexena.MZHeader
import Hexena.PEHeader
import Hexena.Math.Statistics (calculateE)
import Hexena.Internal.SVMWitchcraft

lfanew :: B.ByteString -> Double
lfanew mz = if lf < 0x800 then fromIntegral lf / 0x800 else 1.0
    where lf = runGet getWord32be $ L.fromChunks [B.drop 60 mz]

numberOfSections :: PEHeader -> Double
numberOfSections pe = fromIntegral (min sects 32) / 32.0
    where sects = peNumberOfSections pe

textFormat :: Double -> String
textFormat x = if x < 0 then "MALICIOUS" else "benign"

-- | Create a single line for the libsvm-tools.
runner :: Maybe Model -> String -> IO ()
runner Nothing _ = error "Could not load SVM model!"
runner (Just model) fn = do iff <- loadMalwIFF fn
                            let mz = content $ fromJust $ M.lookup "MZHD" iff
                                mz' = content $ fromJust $ M.lookup "MZRW" iff
                                mzDist = mzDistanceN $ B.unpack mz'
                                (hist, histC) = loadFORMHIST iff
                                entr1 = calculateE hist
                                entr1C = calculateE histC
                                vals = [ numberOfSections $ loadPEHD iff
                                       , mzDist
                                       , lfanew mz
                                       , entr1 / 8.0
                                       , entr1C / 8.0
                                       ]
                                vect = IM.fromList $ zip [1..] vals
                            r <- predict model vect
                            putStrLn $ printf "%s:\t%16.8e\t%s" fn r $ textFormat r


-- One-class SVM classification from hexena files.
{-- main :: IO ()
main = do
  pars <- getArgs
  model <- getPath "svm.vostro7.model" >>= loadModel
  mapM_ (\f -> C.catch (run model f) $
               \e -> hPutStrLn stderr $
                     printf "Exception in handling of '%s': %s" f $
                            show (e::C.SomeException)) pars
--}

run :: FilePath -> Maybe FilePath -> IO ()
run fn Nothing = getDataFileName "svm.filex.2-class-nu.polynomial_nu=0.24_degree=4_gamma=2.1.model" >>=
                 loadModel >>=
                 flip runner fn
run fn (Just modelName) = hPutStrLn stderr ("Warning! Using model in " ++ modelName) >>
                          loadModel modelName >>=
                          flip runner fn
